MAX_X = 500
MAX_Y = 500
VISION_RANGE = 500
ATTACK_RANGE = 100
ATTACK_RELOAD = 0.5
PROJECTILE_DAMAGE = 25
PROJECTILE_VELOCITY = 5
ENEMY_VELOCITY = 4
WALL = -1
FLOOR = 0
STONE = 1
FREE_SPACE = 0
MAP_SCALE = 100
PLAYER_START_DAMAGE = 100
PLAYER_HEALTH = 100
PLAYER_VELOCITY = 15
PORT = 8080
WIDTH = 100
HEIGHT = 100
MIN_ROOM_LEN = 10
MAX_ROOM_LEN = 10
ROOMS = 2
RANDOM_CONNECTIONS = 0
ENEMY_BOX_SIZE = 50
# TODO: нормальный box size
PLAYER_HEIGHT = MAP_SCALE // 2
PLAYER_WIDTH = MAP_SCALE // 2
PLAYER_RELOAD = 0.4

MOVE_UPDATE = 100
PIXEL_SCALE = 100
